@@ -313,7 +313,7 @@ static int supported_features(struct audio_device *device, const char *buf)
 
 	print_hf_features(hs->hf_features);
 
-	err = headset_send(hs, "\r\n+BRSF:%u\r\n", ag.features);
+	err = headset_send(hs, "\r\n+BRSF: %u\r\n", ag.features);
 	if (err < 0)
 		return err;
 
@@ -325,7 +325,7 @@ static char *indicator_ranges(const struct indicator *indicators)
 	int i;
 	GString *gstr;
 
-	gstr = g_string_new("\r\n+CIND:");
+	gstr = g_string_new("\r\n+CIND: ");
 
 	for (i = 0; indicators[i].desc != NULL; i++) {
 		if (i == 0)
@@ -348,7 +348,7 @@ static char *indicator_values(const struct indicator *indicators)
 	int i;
 	GString *gstr;
 
-	gstr = g_string_new("\r\n+CIND:");
+	gstr = g_string_new("\r\n+CIND: ");
 
 	for (i = 0; indicators[i].desc != NULL; i++) {
 		if (i == 0)
@@ -501,7 +501,7 @@ static gboolean ring_timer_cb(gpointer data)
 
 	if (ag.number)
 		send_foreach_headset(active_devices, cli_cmp,
-					"\r\n+CLIP:\"%s\",%d\r\n",
+					"\r\n+CLIP: \"%s\",%d\r\n",
 					ag.number, ag.number_type);
 
 	return TRUE;
@@ -630,7 +630,7 @@ static int telephony_generic_rsp(struct audio_device *device, cme_error_t err)
 
 	if (err != CME_ERROR_NONE) {
 		if (hs->cme_enabled)
-			return headset_send(hs, "\r\n+CME ERROR:%d\r\n", err);
+			return headset_send(hs, "\r\n+CME ERROR: %d\r\n", err);
 		else
 			return headset_send(hs, "\r\nERROR\r\n");
 	}
@@ -709,7 +709,7 @@ static int call_hold(struct audio_device *dev, const char *buf)
 		return 0;
 	}
 
-	err = headset_send(hs, "\r\n+CHLD:(%s)\r\n", ag.chld);
+	err = headset_send(hs, "\r\n+CHLD: (%s)\r\n", ag.chld);
 	if (err < 0)
 		return err;
 
@@ -826,7 +826,7 @@ static int response_and_hold(struct audio_device *device, const char *buf)
 	}
 
 	if (ag.rh >= 0)
-		headset_send(hs, "\r\n+BTRH:%d\r\n", ag.rh);
+		headset_send(hs, "\r\n+BTRH: %d\r\n", ag.rh);
 
 	return headset_send(hs, "\r\nOK\r\n", ag.rh);
 }
@@ -1007,12 +1007,17 @@ int telephony_call_hold_rsp(void *telephony_device, cme_error_t err)
 	return telephony_generic_rsp(telephony_device, err);
 }
 
+int telephony_disable_nr_and_ec_rsp(void *telephony_device, cme_error_t err)
+{
+	return telephony_generic_rsp(telephony_device, err);
+}
+
 int telephony_operator_selection_ind(int mode, const char *oper)
 {
 	if (!active_devices)
 		return -ENODEV;
 
-	send_foreach_headset(active_devices, hfp_cmp, "\r\n+COPS:%d,0,%s\r\n",
+	send_foreach_headset(active_devices, hfp_cmp, "\r\n+COPS: %d,0,%s\r\n",
 				mode, oper);
 	return 0;
 }
@@ -1037,6 +1042,16 @@ static int operator_selection(struct audio_device *device, const char *buf)
 	return 0;
 }
 
+static int disable_nr_and_ec(struct audio_device *device, const char *buf)
+{
+	if (strlen(buf) < 9)
+		return -EINVAL;
+
+	telephony_disable_nr_and_ec_req(device);
+
+	return 0;
+}
+
 static struct event event_callbacks[] = {
 	{ "ATA", answer_call },
 	{ "ATD", dial_number },
@@ -1056,6 +1071,7 @@ static struct event event_callbacks[] = {
 	{ "AT+CMEE", extended_errors },
 	{ "AT+CCWA", call_waiting_notify },
 	{ "AT+COPS", operator_selection },
+	{ "AT+NREC", disable_nr_and_ec },
 	{ 0 }
 };
 
@@ -2105,7 +2121,9 @@ static gboolean dummy_connect_complete(struct audio_device *dev)
 	return FALSE;
 }
 
-unsigned int headset_request_stream(struct audio_device *dev, headset_stream_cb_t cb,
+unsigned int headset_request_stream(struct audio_device *dev,
+					headset_stream_cb_t cb,
+					headset_lock_t lock,
 					void *user_data)
 {
 	struct headset *hs = dev->headset;
@@ -2122,7 +2140,8 @@ unsigned int headset_request_stream(struct audio_device *dev, headset_stream_cb_
 		hs->dc_timer = 0;
 	}
 
-	if (hs->state == HEADSET_STATE_CONNECT_IN_PROGRESS)
+	if (hs->state == HEADSET_STATE_CONNECT_IN_PROGRESS ||
+			hs->state == HEADSET_STATE_PLAY_IN_PROGRESS)
 		return connect_cb_new(hs, HEADSET_STATE_PLAYING, cb, user_data);
 
 	if (hs->rfcomm == NULL) {
@@ -2139,6 +2158,66 @@ unsigned int headset_request_stream(struct audio_device *dev, headset_stream_cb_
 	return id;
 }
 
+unsigned int headset_config_stream(struct audio_device *dev,
+					headset_stream_cb_t cb,
+					headset_lock_t lock,
+					void *user_data)
+{
+	struct headset *hs = dev->headset;
+	unsigned int id;
+
+	if (hs->lock & lock)
+		return 0;
+
+	if (hs->dc_timer) {
+		g_source_remove(hs->dc_timer);
+		hs->dc_timer = 0;
+	}
+
+	if (hs->state == HEADSET_STATE_CONNECT_IN_PROGRESS)
+		return connect_cb_new(hs, HEADSET_STATE_CONNECTED, cb,
+					user_data);
+
+	if (hs->rfcomm)
+		goto done;
+
+	if (rfcomm_connect(dev, cb, user_data, &id) < 0)
+		return 0;
+
+	hs->auto_dc = TRUE;
+	hs->pending->target_state = HEADSET_STATE_CONNECTED;
+
+	return id;
+
+done:
+	id = connect_cb_new(hs, HEADSET_STATE_CONNECTED, cb, user_data);
+	g_idle_add((GSourceFunc) dummy_connect_complete, dev);
+	return id;
+}
+
+unsigned int headset_suspend_stream(struct audio_device *dev,
+					headset_stream_cb_t cb,
+					headset_lock_t lock,
+					void *user_data)
+{
+	struct headset *hs = dev->headset;
+	unsigned int id;
+
+	if (hs->lock & ~lock || !hs->rfcomm || !hs->sco)
+		return 0;
+
+	if (hs->dc_timer) {
+		g_source_remove(hs->dc_timer);
+		hs->dc_timer = 0;
+	}
+
+	close_sco(dev);
+
+	id = connect_cb_new(hs, HEADSET_STATE_CONNECTED, cb, user_data);
+	g_idle_add((GSourceFunc) dummy_connect_complete, dev);
+	return id;
+}
+
 gboolean get_hfp_active(struct audio_device *dev)
 {
 	struct headset *hs = dev->headset;
@@ -2398,7 +2477,7 @@ int telephony_event_ind(int index)
 	}
 
 	send_foreach_headset(active_devices, hfp_cmp,
-				"\r\n+CIEV:%d,%d\r\n", index + 1,
+				"\r\n+CIEV: %d,%d\r\n", index + 1,
 				ag.indicators[index].val);
 
 	return 0;
@@ -2415,7 +2494,7 @@ int telephony_response_and_hold_ind(int rh)
 	if (ag.rh < 0)
 		return 0;
 
-	send_foreach_headset(active_devices, hfp_cmp, "\r\n+BTRH:%d\r\n",
+	send_foreach_headset(active_devices, hfp_cmp, "\r\n+BTRH: %d\r\n",
 				ag.rh);
 
 	return 0;
@@ -2513,12 +2592,12 @@ int telephony_list_current_call_ind(int idx, int dir, int status, int mode,
 
 	if (number)
 		send_foreach_headset(active_devices, hfp_cmp,
-					"\r\n+CLCC:%d,%d,%d,%d,%d,%s,%d\r\n",
+					"\r\n+CLCC: %d,%d,%d,%d,%d,%s,%d\r\n",
 					idx, dir, status, mode, mprty,
 					number, type);
 	else
 		send_foreach_headset(active_devices, hfp_cmp,
-					"\r\n+CLCC:%d,%d,%d,%d,%d\r\n",
+					"\r\n+CLCC: %d,%d,%d,%d,%d\r\n",
 					idx, dir, status, mode, mprty);
 
 	return 0;
@@ -2530,7 +2609,7 @@ int telephony_subscriber_number_ind(const char *number, int type, int service)
 		return -ENODEV;
 
 	send_foreach_headset(active_devices, hfp_cmp,
-				"\r\n+CNUM:,%s,%d,,%d\r\n",
+				"\r\n+CNUM: ,%s,%d,,%d\r\n",
 				number, type, service);
 
 	return 0;
@@ -2552,7 +2631,7 @@ int telephony_call_waiting_ind(const char *number, int type)
 	if (!active_devices)
 		return -ENODEV;
 
-	send_foreach_headset(active_devices, cwa_cmp, "\r\n+CCWA:%s,%d\r\n",
+	send_foreach_headset(active_devices, cwa_cmp, "\r\n+CCWA: %s,%d\r\n",
 				number, type);
 
 	return 0;
