@@ -101,11 +101,7 @@ static void agent_release(struct agent *agent)
 		return;
 	}
 
-	dbus_message_set_no_reply(message, TRUE);
-
-	dbus_connection_send(connection, message, NULL);
-
-	dbus_message_unref(message);
+	g_dbus_send_message(connection, message);
 }
 
 static int send_cancel_request(struct agent_request *req)
@@ -119,11 +115,7 @@ static int send_cancel_request(struct agent_request *req)
 		return -ENOMEM;
 	}
 
-	dbus_message_set_no_reply(message, TRUE);
-
-	dbus_connection_send(connection, message, NULL);
-
-	dbus_message_unref(message);
+	g_dbus_send_message(connection, message);
 
 	return 0;
 }
