@@ -45,6 +45,9 @@
 #define HCID_OFFMODE_DEVDOWN	0
 #define HCID_OFFMODE_NOSCAN	1
 
+/* Timeout for hci_send_req (milliseconds) */
+#define HCI_REQ_TIMEOUT		5000
+
 struct main_opts {
 	char		host_name[40];
 	unsigned long	flags;
@@ -52,6 +55,7 @@ struct main_opts {
 	uint32_t	class;
 	uint16_t	pageto;
 	uint32_t	discovto;
+	uint32_t	pairto;
 	uint16_t	link_mode;
 	uint16_t	link_policy;
 
