@@ -48,10 +48,17 @@
 #include "error.h"
 #include "manager.h"
 
+static char base_path[50] = "/org/bluez";
+
 static DBusConnection *connection = NULL;
 static int default_adapter_id = -1;
 static GSList *adapters = NULL;
 
+const char *manager_get_base_path(void)
+{
+	return base_path;
+}
+
 int manager_update_adapter(uint16_t dev_id)
 {
 	struct btd_adapter *adapter;
@@ -182,7 +189,12 @@ static DBusMessage *find_adapter(DBusConnection *conn,
 
 	/* hci_devid() would make sense to use here, except it
 	   is restricted to devices which are up */
-	if (!strncmp(pattern, "hci", 3) && strlen(pattern) >= 4)
+	if (!strcmp(pattern, "any") || !strcmp(pattern, "00:00:00:00:00:00")) {
+		path = adapter_any_get_path();
+		if (path != NULL)
+			goto done;
+		dev_id = -1;
+	} if (!strncmp(pattern, "hci", 3) && strlen(pattern) >= 4)
 		dev_id = atoi(pattern + 3);
 	else
 		dev_id = find_by_address(pattern);
@@ -200,14 +212,15 @@ static DBusMessage *find_adapter(DBusConnection *conn,
 	if (!adapter)
 		return no_such_adapter(msg);
 
+	path = adapter_get_path(adapter);
+
+done:
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
 		return NULL;
 
-	path = adapter_get_path(adapter);
-
 	dbus_message_append_args(reply, DBUS_TYPE_OBJECT_PATH, &path,
-				DBUS_TYPE_INVALID);
+							DBUS_TYPE_INVALID);
 
 	return reply;
 }
@@ -269,6 +282,8 @@ dbus_bool_t manager_init(DBusConnection *conn, const char *path)
 {
 	connection = conn;
 
+	snprintf(base_path, sizeof(base_path), "/org/bluez/%d", getpid());
+
 	return g_dbus_register_interface(conn, "/", MANAGER_INTERFACE,
 			manager_methods, manager_signals,
 			NULL, NULL, NULL);
@@ -361,6 +376,13 @@ static void manager_add_adapter(struct btd_adapter *adapter)
 			DBUS_TYPE_INVALID);
 
 	adapters = g_slist_append(adapters, adapter);
+
+	if (default_adapter_id < 0) {
+		int new_default = hci_get_route(NULL);
+
+		if (new_default >= 0)
+			manager_set_default_adapter(new_default);
+	}
 }
 
 static void manager_remove_adapter(struct btd_adapter *adapter)
@@ -373,7 +395,7 @@ static void manager_remove_adapter(struct btd_adapter *adapter)
 			DBUS_TYPE_OBJECT_PATH, &path,
 			DBUS_TYPE_INVALID);
 
-	if ((default_adapter_id == dev_id || default_adapter_id < 0)) {
+	if (default_adapter_id == dev_id || default_adapter_id < 0) {
 		int new_default = hci_get_route(NULL);
 
 		if (new_default >= 0)
@@ -430,7 +452,7 @@ int manager_start_adapter(int id)
 	if (ret < 0)
 		return ret;
 
-	if (manager_get_default_adapter() < 0)
+	if (default_adapter_id < 0)
 		manager_set_default_adapter(id);
 
 	return 0;
